Package: legendry
Title: Extended Legends and Axes for 'ggplot2'
Version: 0.3.0
Authors@R: 
    person("Teun", "van den Brand", , "tahvdbrand@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-9335-7468"))
Description: A 'ggplot2' extension that focusses on expanding the
    plotter's arsenal of guides. Guides in 'ggplot2' include axes and
    legends. 'legendry' offers new axes and annotation options, as well as
    new legends and colour displays.
License: MIT + file LICENSE
URL: https://teunbrand.github.io/legendry/,
        https://github.com/teunbrand/legendry
BugReports: https://github.com/teunbrand/legendry/issues
Depends: ggplot2 (>= 4.0.0), R (>= 4.1.0)
Imports: cli, grid (>= 4.1.0), gtable, lifecycle, rlang (>= 1.2.0), S7,
        scales (>= 1.1.1), vctrs (>= 0.6.0)
Suggests: knitr, ragg, rmarkdown, svglite, testthat (>= 3.0.0), vdiffr,
        withr
Config/testthat/edition: 3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-12 19:00:54 UTC; Teun
Author: Teun van den Brand [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-9335-7468>)
Maintainer: Teun van den Brand <tahvdbrand@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-13 05:10:25 UTC
Built: R 4.5.2; ; 2026-05-13 09:56:42 UTC; unix
