# Generated by roxygen2: do not edit by hand

export(Compose)
export(ComposeCrux)
export(ComposeOntop)
export(ComposeSandwich)
export(ComposeStack)
export(GizmoBarcap)
export(GizmoDensity)
export(GizmoGrob)
export(GizmoHistogram)
export(GizmoStepcap)
export(GuideAxisAnnotation)
export(GuideAxisPlot)
export(GuideAxisSymbols)
export(GuideCircles)
export(GuideColring)
export(GuideLegendBase)
export(GuideLegendGroup)
export(GuideLegendManual)
export(PrimitiveBox)
export(PrimitiveBracket)
export(PrimitiveFence)
export(PrimitiveLabels)
export(PrimitiveLine)
export(PrimitiveSpacer)
export(PrimitiveTicks)
export(PrimitiveTitle)
export(annotate_bottom)
export(annotate_left)
export(annotate_right)
export(annotate_top)
export(bracket_atan)
export(bracket_chevron)
export(bracket_curvy)
export(bracket_line)
export(bracket_round)
export(bracket_sigmoid)
export(bracket_square)
export(cap_arch)
export(cap_none)
export(cap_ogee)
export(cap_round)
export(cap_triangle)
export(compose_crux)
export(compose_ontop)
export(compose_sandwich)
export(compose_stack)
export(gizmo_barcap)
export(gizmo_density)
export(gizmo_grob)
export(gizmo_histogram)
export(gizmo_stepcap)
export(guide_axis_annotation)
export(guide_axis_base)
export(guide_axis_dendro)
export(guide_axis_nested)
export(guide_axis_plot)
export(guide_axis_symbols)
export(guide_axis_upset)
export(guide_circles)
export(guide_colbar)
export(guide_colring)
export(guide_colsteps)
export(guide_legend_base)
export(guide_legend_cross)
export(guide_legend_group)
export(guide_legend_manual)
export(key_auto)
export(key_bins)
export(key_dendro)
export(key_group_lut)
export(key_group_split)
export(key_log)
export(key_manual)
export(key_map)
export(key_minor)
export(key_none)
export(key_range_auto)
export(key_range_manual)
export(key_range_map)
export(key_range_rle)
export(key_segment_manual)
export(key_segment_map)
export(key_sequence)
export(key_symbols)
export(key_upset)
export(new_compose)
export(position_text)
export(primitive_box)
export(primitive_bracket)
export(primitive_fence)
export(primitive_labels)
export(primitive_line)
export(primitive_segments)
export(primitive_spacer)
export(primitive_ticks)
export(primitive_title)
export(scale_x_dendro)
export(scale_y_dendro)
export(theme_guide)
import(ggplot2)
import(grid)
import(gtable)
import(rlang)
import(scales)
import(vctrs)
importFrom(lifecycle,deprecated)
