
#' @name guide-gizmos
#' @title Guide gizmos
#'
#' @description
#' Guide gizmos are a speciality guide components that are very specific to
#' one or a few aesthetics to display.
#'
#' Typically they can be [composed][guide-composition] with other guides or
#' guide [primitives][guide-primitives] to form a complete guide.
NULL
