# connectors works

    The value_start, value_end, level_start, and level_end columns are required in `connect`.

# guide_axis_symbols does input checks

    The `key` argument is required. You can set a manual key using `key_symbols()`.

# guide_axis_upset treats input correctly

    The `key` argument is required. You can set a key using `key_upset()`.

# guide_axis_upset gives informative error for `override.aes`

    Can't recycle `override.aes$colour` (size 4) to size 3.

