# key_group_split works correctly

    Code
      key_group_split(sep = ":")(scale, "colour")
    Condition
      Error in `key_group_split()`:
      ! Cannot split the guide's label.
      i It must be a <character> vector, not a list.

