# range_from_label can extract ranges

    Code
      test
    Output
        start end .label .level
      1     1   1      A      0
      2     2   2      B      0
      3     3   3      C      0
      4     1   2      1      1
      5     3   3      2      1

