% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-.R
\name{guide-primitives}
\alias{guide-primitives}
\title{Guide primitives}
\description{
Guide primitives are the building blocks of more complex guides. On their
own, they are not very useful as a visual reflection of a scale.

Their purpose is to be combined with one another to form a more complex,
complete guides that \emph{do} reflect a scale in some way.
}
\details{
The guide primitives are simple, but flexible in that they are not tailored
for one particular aesthetic. That way they can be reused and combined
at will.
}
