% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legendry-package.R
\docType{package}
\name{legendry-package}
\alias{legendry}
\alias{legendry-package}
\title{legendry: Extended Legends and Axes for 'ggplot2'}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A 'ggplot2' extension that focusses on expanding the plotter's arsenal of guides. Guides in 'ggplot2' include axes and legends. 'legendry' offers new axes and annotation options, as well as new legends and colour displays.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://teunbrand.github.io/legendry/}
  \item \url{https://github.com/teunbrand/legendry}
  \item Report bugs at \url{https://github.com/teunbrand/legendry/issues}
}

}
\author{
\strong{Maintainer}: Teun van den Brand \email{tahvdbrand@gmail.com} (\href{https://orcid.org/0000-0002-9335-7468}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Teun van den Brand \email{tahvdbrand@gmail.com} (\href{https://orcid.org/0000-0002-9335-7468}{ORCID}) [copyright holder]
}

}
\keyword{internal}
