% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gizmo-.R
\name{guide-gizmos}
\alias{guide-gizmos}
\title{Guide gizmos}
\description{
Guide gizmos are a speciality guide components that are very specific to
one or a few aesthetics to display.

Typically they can be \link[=guide-composition]{composed} with other guides or
guide \link[=guide-primitives]{primitives} to form a complete guide.
}
