% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compose-.R, R/compose-crux.R,
%   R/compose-ontop.R, R/compose-sandwich.R, R/compose-stack.R,
%   R/gizmo-barcap.R, R/gizmo-density.R, R/gizmo-grob.R, R/gizmo-histogram.R,
%   R/gizmo-stepcap.R, R/guide-axis-annotation.R, R/guide-axis-plot.R,
%   R/guide-axis-symbols.R, R/guide-circles.R, R/guide-colring.R,
%   R/guide-legend-base.R, R/guide-legend-group.R, R/guide-legend-manual.R,
%   R/legendry-package.R, R/primitive-box.R, R/primitive-bracket.R,
%   R/primitive-fence.R, R/primitive-labels.R, R/primitive-line.R,
%   R/primitive-spacer.R, R/primitive-ticks.R, R/primitive-title.R
\name{Compose}
\alias{Compose}
\alias{ComposeCrux}
\alias{ComposeOntop}
\alias{ComposeSandwich}
\alias{ComposeStack}
\alias{GizmoBarcap}
\alias{GizmoDensity}
\alias{GizmoGrob}
\alias{GizmoHistogram}
\alias{GizmoStepcap}
\alias{GuideAxisAnnotation}
\alias{GuideAxisPlot}
\alias{GuideAxisSymbols}
\alias{GuideCircles}
\alias{GuideColring}
\alias{GuideLegendBase}
\alias{GuideLegendGroup}
\alias{GuideLegendManual}
\alias{legendry_extensions}
\alias{PrimitiveBox}
\alias{PrimitiveBracket}
\alias{PrimitiveFence}
\alias{PrimitiveLabels}
\alias{PrimitiveLine}
\alias{PrimitiveSpacer}
\alias{PrimitiveTicks}
\alias{PrimitiveTitle}
\title{\code{ggproto} objects in \pkg{legendry}}
\description{
The \pkg{legendry} package relies on an extension system of
\pkg{ggplot2} through \code{\link[ggplot2:ggproto]{ggproto}} class objects, which
allow cross-package inheritance of objects such as geoms, stats, facets,
scales and coordinate systems. For the purpose of making plots, users are
invited to wholly ignore these objects, since interacting with these
objects is preferred through various constructor functions. The
\pkg{legendry} package introduces a new \verb{<Guide>} ggproto class to support
variations on axes, legends and colourbars.
}
\seealso{
The documentation over at \code{\link[ggplot2:ggproto]{ggproto}}.
}
\keyword{internal}
